@@ -1,4 +1,4 @@
-"""Contains the DrillString class
+"""A module that contains the :class:`DrillString` class
 """
 
 import os
@@ -18,7 +18,7 @@ class DrillString():
 
     Note
     ----
-    Once the DrillString is instanced tools within the string must be defined as DrillTool objects before the string is written to a string file.  Use the `add_tool()` method to add DrillTool objects to the string.
+    Once the :class:`DrillString` is instanced, tools within the string must be defined as :class:`DrillTool` objects before the string is written to a string file.  Use the :meth:`add_tool()` method to add :class:`DrillTool` objects to the string.
 
     Attributes
     ----------
@@ -43,7 +43,7 @@ class DrillString():
     MULTI_JOINT_TOOLS : list
         A class attribute listing all the tool types that need to have `Number_of_Joints` defined in the Adams Drill string file.
     DRILL_TOOL_PATTERN : SRE_Pattern
-        A :class:`_sre.SRE_Pattern` object defining the pattern of the Drill Tool block header in the string file.    
+        A :obj:`_sre.SRE_Pattern` object defining the pattern of the Drill Tool block header in the string file.    
     """
 
     SCALAR_PARAMETERS = [
@@ -102,27 +102,34 @@ class DrillString():
 
     def add_tool(self, drill_tool, joints=1, measure=False, stack_order=None, color='Default', group_name=None, equivalent=False):
         """
-        Adds a DrillTool object to the DrillString.
+        Adds a :class:`DrillTool` object to the :class:`DrillString`.
+
+        Note
+        ----
+        You cannot add the same :class:`DrillTool` object to the string multiple times.  If you want to add multiple instances of the same tool you must create two seperate :class:`DrillTool` objects from the same Tiem Orbit property file.
         
         Parameters
         ----------
         tool : DrillTool
-            DrillTool object representing the tool to be added        
+            :class:`DrillTool` object representing the tool to be added        
         joints : int
             Number of Joints. Note that this only applies for certain tool types. (default is 1)
         measure : bool
             If TRUE indicates that output requests should be generated for this tool.  (default is FALSE)
         stack_order : int
-            If an integer is given the tool will be inserted into the string at that point. (default is None and the tool will be appended to the end)
+            If an integer is given the tool will be inserted into the string at that point. (default is None which means the tool will be appended to the end)
         color : str
             The color used to render the tool in an Adams Drill animation. (default is 'Default')
         """
         # Check that the group_name argument is givn
         if drill_tool.tool_type.lower() in self.MULTI_JOINT_TOOLS and group_name is None:
             raise ValueError('group_name is required for tools of type {}.'.format('{}'.format(self.MULTI_JOINT_TOOLS)[1:-1].replace("'",'')))
-        
+                
         if drill_tool.tool_type.lower() != 'top_drive':
-            # If the tool added IS NOT a top_drive
+            # If the tool added IS NOT a top_drive, check that the tool is not already in the tools list
+            for existing_tool in self.tools:
+                if drill_tool is existing_tool:
+                    raise DrillStringError('You cannot add the same DrillTool object to a DrillString multiple times!')
 
             # Create a dictionary describing the tool
             tool = {
@@ -165,26 +172,25 @@ class DrillString():
                 'Name': drill_tool.name,
                 'Property_File': drill_tool.property_file
             }
+        
+        # This binds drill_tool.name to tool['Name'] and tool['Property_File]
+        # Now if drill_tool is renamed by another processes, the tools list won't need to be updated.
+        drill_tool.bind_name_to(self.tool_renamed)
     
     def set_pipe_joints(self, joints, equivalent=False):
-        """Sets the number of joints in the upper most section of
-        drill pipe. Set pipe_type='equivalent' to adjust equivalent
-        upper string joints'
+        """Sets the number of joints in the upper most section of drill pipe. Set `equivalent=True` to adjust equivalent upper string joints.
         
         Parameters
         ----------
-            joints {int}           -- Number of physical drill 
-                                      pipe joints.
-            equivalent {str}       -- False for physical
-                                      string. True for 
-                                      equivalent upper string.
-                                      Default=False
+        joints : int
+            Number of physical drill pipe joints.
+        equivalent : str
+            False for physical string. True for equivalent upper string. (Default is False.)
         
         Raises
         ------
-            DrillStringError       -- Raised if the drill string
-                                      doesn't have drill pipe 
-                                      of the specified type.
+        DrillStringError
+            Raised if the drill string doesn't have drill pipe of the specified type.
         """
         # Check that the pipe type argument was passed correctly
 
@@ -202,8 +208,7 @@ class DrillString():
             raise DrillStringError(f'There is no {tool_type} in this string!')        
     
     def get_tool(self, tool_type, index=0):
-        """Returns a DrillTool object of type tool_type in the 
-        DrillString object's tools list.  
+        """Returns a DrillTool object of type :arg:`tool_type` in the :class:`DrillString` object's tools list.  
         
         Parameters
         ----------
@@ -216,12 +221,12 @@ class DrillString():
         Raises
         ------
         DrillStringError
-            Raised if a tools of the specified type does not exist in the drill string.
+            Raised if a tool of the specified type does not exist in the drill string.
         
         Returns
         -------
         DrillTool
-            DrillTool object
+            :class:`DrillTool` object
         """
         tools_found = []
         for tool in self.tools:
@@ -243,13 +248,29 @@ class DrillString():
 
         return tools_found[index]
         
-    def get_bha_length(self):
+    def tool_renamed(self, renamed_tool):
+        """Updates the 'name' and 'Property_File' entries in the :attr:`tools` to match :arg:`renamed_tool`.name.        
+
+        Parameters
+        ----------
+        renamed_tool : DrillTool
+            :class:`DrillTool` object that has been renamed and needs to have its 'Name' and 'Property_File' entries updates.
         """
-        Gets the total length of the drill string defined in 
-        string_file NOT including the equivalent upper string and 
-        highest most physical string.
+
+        for tool in self.tools:
+            if tool['DrillTool'] is renamed_tool:
+                tool['Name'] = renamed_tool.name
+                ext = tool['DrillTool'].extension
+                tool['Property_File'] = os.path.join(os.path.split(tool['Property_File'])[0], f'{renamed_tool.name}.{ext}')
+                break
+
+    def get_bha_length(self):
+        """Gets the total length of the BHA defined in string_file 
         
-                        
+        Note
+        ----
+        The BHA is all the tools EXCEPT the equivalent upper string and highest most physical string.
+                                
         Returns
         -------
         float : Cumulative length of the bha    
@@ -278,8 +299,10 @@ class DrillString():
         """
         Adds a measurement point at the given distance from the bit in feet.
         
-        Arguments:
-            distance {float} -- Distance from the bit (in feet) at which to place a measurement point.
+        Parameters
+        ----------
+        distance : float
+            Distance from the bit (in feet) at which to place a measurement point.
         """
         self.parameters['Distance_from_Bit'].append(distance)
         self.parameters['Distance_from_Bit'].sort()
@@ -301,6 +324,8 @@ class DrillString():
             Name of the cdb in which to write the file.  This argument is overridden by `directory`. (default is None which means the string will be written to the directory in `directory` and will use the filename in `filename`.)  
         publish : bool
             If true, writes all the supporting files to the same cdb. (default is False.)
+        publish_event : bool
+
         
         Raises
         ------
@@ -361,7 +386,9 @@ class DrillString():
         if publish is True:
             # If the string is being published, copy all the tools to the new 
             # cdb or directory
-            for tool in self.tools:      
+            for tool in self.tools:   
+
+                # Copy the DrillTool object   
                 tool['DrillTool'].copy_file(cdb=cdb, directory=directory)
 
                 # Update the tool file paths in the drill_string.tools list
@@ -414,6 +441,33 @@ class DrillString():
         
         return get_cdb_path(filepath)
     
+    @classmethod
+    def read_from_file(cls, filename):
+        """Reads a string file and returns a :class:`DrillString` object with `DrillString.parameters` based on data in the string file.
+        
+        Parameters
+        ----------
+        filename : str
+            Filename of a string file.
+            
+        Returns
+        -------
+        DrillString
+            :class:`DrillString` object with parameters from the passed string file.
+        """
+        # Read the TO data into a dictionary
+        tiem_orbit_data = read_TO_file(get_full_path(filename))
+
+        drill_string = cls('','','')
+        
+        # Extract the DrillString parameters from the TO dictionary        
+        drill_string._get_params_from_TO_data(tiem_orbit_data) #pylint: disable=protected-access
+
+        # Extract the DrillTools from the TO dictionary
+        drill_string._get_tools_from_TO_data(tiem_orbit_data) #pylint: disable=protected-access
+
+        return drill_string
+    
     def validate(self):
         """
         Determines if all parameters have been set
@@ -456,32 +510,6 @@ class DrillString():
             validation['reason'] = 'the string doesn\'t have a bit'     
 
         return validation   
-    
-    @classmethod
-    def read_from_file(cls, filename):
-        """Reads a string file and returns a DrillString object
-        with DrillString.parameters based on data in the string 
-        file.
-        
-        Arguments:
-            filename {str} -- Filename of a .str file.
-            
-        Returns:
-            {DrillString} -- DrillString object with parameters
-                             from the passed string file.
-        """
-        # Read the TO data into a dictionary
-        tiem_orbit_data = read_TO_file(get_full_path(filename))
-
-        drill_string = cls('','','')
-        
-        # Extract the DrillString parameters from the TO dictionary        
-        drill_string._get_params_from_TO_data(tiem_orbit_data) #pylint: disable=protected-access
-
-        # Extract the DrillTools from the TO dictionary
-        drill_string._get_tools_from_TO_data(tiem_orbit_data) #pylint: disable=protected-access
-
-        return drill_string
 
     def _apply_defaults(self):
         """
@@ -501,9 +529,12 @@ class DrillString():
     def _copy_hole_file(self, directory=None, cdb=None):
         """Copys the hole file to a new locations.  Must give directory OR cdb.
         
-        Keyword Arguments:
-            directory {string} -- Directory to put the copied hole file (default: {None})
-            cdb {string} -- Name of cdb to put the copied hole file (default: {None})
+        Parameters
+        ----------
+        directory : str
+            Directory to put the copied hole file (default is None)
+        cdb : str
+            Name of cdb to put the copied hole file (default is None)
         """
         # Check that directory or cdb was given.
         if cdb is None and directory is None:
@@ -525,11 +556,18 @@ class DrillString():
         self.parameters['Hole_Property_File'] = get_cdb_path(new_filename)
     
     def _copy_event_file(self, directory=None, cdb=None):
-        """Copys the event file to a new locations.  Must give directory OR cdb.
+        """Copys the event file to a new locations.
         
-        Keyword Arguments:
-            directory {string} -- Directory to put the copied event file (default: {None})
-            cdb {string} -- Name of cdb to put the copied event file (default: {None})
+        Parameters
+        ----------
+        directory : str
+            Directory to put the copied event file (default is None)
+        cdb : str
+            Name of cdb to put the copied event file (default is None)
+        
+        Note
+        ----
+        Either directory or cdb must be given.
         """
         # Check that directory or cdb was given.
         if cdb is None and directory is None:
@@ -551,11 +589,12 @@ class DrillString():
         self.parameters['Event_Property_File'] = get_cdb_path(new_filename)
     
     def _get_tools_from_TO_data(self, tiem_orbit_data):
-        """Reads the tools in the string out of a dictoinary of Tiem Orbit
-        data generated by adripy.utilities.read_TO_file()
+        """Reads the tools in the string out of a dictoinary of Tiem Orbit data generated by `adripy.utilities.read_TO_file()`
         
-        Arguments:
-            tiem_orbit_data {dict} -- Dictionary of Tiem Orbit data
+        Parameters
+        ----------
+        tiem_orbit_data : dict
+            Dictionary of Tiem Orbit data
         """
         # Add the tools
         drill_tool_blocks = [block for block in tiem_orbit_data if self.DRILL_TOOL_PATTERN.match(block)]
@@ -563,7 +602,7 @@ class DrillString():
             # For each drill tool block, create a DrillTool object
             tool = DrillTool(tiem_orbit_data[block]['property_file'])
 
-            # Check if the tool as a number_of_joints parameter
+            # Check if the tool has a number_of_joints parameter
             if 'number_of_joints' in tiem_orbit_data[block]:
                 joints = tiem_orbit_data[block]['number_of_joints']
                 group_name = tiem_orbit_data[block]['name']
@@ -596,7 +635,6 @@ class DrillString():
         ValueError
             A string parameter could not be found
         """
-
         for param in self.SCALAR_PARAMETERS + self.ARRAY_PARAMETERS:
             # For each string parameter initialize a found flag
             found = False
@@ -628,8 +666,8 @@ class DrillString():
                         break
             
             # Raise a value error if the parameter isn't found.
-            if not found and param is not 'Distance_from_Bit':
+            if not found and param != 'Distance_from_Bit':
                 raise ValueError(f'{param} not found!')
 
 class DrillStringError(Exception):
-    pass
\ No newline at end of file
+    pass
