@@ -202,7 +202,7 @@ def get_adrill_cdbs(adrill_user_cfg, adrill_shared_cfg=None):
     ----------
     adrill_user_cfg : str
         Full path to an Adams Drill user configuration file.  This hould be in the users HOME directory.
-    adrill_shared_cfg : :obj:`str`, optional
+    adrill_shared_cfg : str
         Full path to an Adams Drill shared configuration file.  This should be in the Adams Drill installation directory.  (the default is None, which means that only user cdbs will be returned.)
         
     Returns
@@ -623,7 +623,7 @@ def get_string_length(string_file):
                       
     Returns
     -------
-    string_length : str
+    float
         Cumulative length of the string    
     """
     cdbs = get_adrill_cdbs(os.environ['ADRILL_USER_CFG'], os.environ['ADRILL_SHARED_CFG'])
@@ -893,6 +893,15 @@ def build(string_file, solver_settings_file, working_directory, output_name=None
         Path to the directory to put the adm, acf, and cmd
     output_name : str
         Base name of the adm, acf, and cmd files. (the default is none, which redefines the `ouput_name` to be the same as the 'OutputName' parameter in the string file)
+
+    Returns
+    -------
+    str
+        Filename of the adams dataset (.adm) file.
+    str
+        Filename of the adams command (.acf) file.
+    str
+        Filename of the adams view command (.cmd) file.
     """    
     # Set the output name  
     if output_name is None:  
@@ -937,6 +946,10 @@ def build(string_file, solver_settings_file, working_directory, output_name=None
     process = subprocess.Popen('{} aview ru-s b build.cmd'.format(os.environ['ADAMS_LAUNCH_COMMAND']), cwd=working_directory, startupinfo=startupinfo)
     process.wait()    
 
+    adm_file = os.path.join(working_directory, adm_file)
+    acf_file = os.path.join(working_directory, acf_file)
+    cmd_file = os.path.join(working_directory, cmd_file)
+
     return adm_file, acf_file, cmd_file
 
 TO_PARAMETER_PATTERN = re.compile('^ +[_0-9a-zA-Z]+\\s+=\\s+((\'[-:_0-9a-zA-Z<>\\\\/\\.]*\')|(-?[\\+-\\.e0-9]+))\\s*$')
@@ -948,11 +961,15 @@ TO_TABLE_LINE_PATTERN = re.compile("^((\\s*\\'[_0-9a-zA-Z]+\\')+)|((\\s*-?[\\+-\
 def read_TO_file(filename):
     """Reads a Tiem Orbit file into a dictionary of parameters
     
-    Arguments:
-        filename {string} -- Filename of the Tiem Orbit file
+    Parameters
+    ----------
+    filename : str
+        Filename of the Tiem Orbit file
     
-    Raises:
-        TiemOrbitSyntaxError -- Raised if the Tiem Orbit syntax is not recognized
+    Raises
+    ------
+    TiemOrbitSyntaxError
+        Raised if the Tiem Orbit syntax is not recognized
     """
     filename = get_full_path(os.path.normpath(filename))
     
